@@ -6,10 +6,8 @@ from enum import Enum
 from itertools import chain
 from numbers import Integral
 from typing import (
-    TypeVar,
     Tuple,
     FrozenSet,
-    Union,
     Set,
     Iterable,
     Any,
@@ -20,27 +18,12 @@ from typing import (
     Optional,
     Iterator,
     Sequence,
+    Type,
 )
 
-from smallgraphlib.utilities import (
-    ComparableAndHashable,
-    cached_property,
-    Multiset,
-    clear_cache,
-    segments_intersection,
-)
-
-_TIKZ_EXPORT_MAX_MULTIPLE_EDGES_SUPPORT = 3
-_TIKZ_EXPORT_MAX_MULTIPLE_LOOPS_SUPPORT = 1
-
-# Node = TypeVar("Node", bound=typing.Hashable)  # too subtile for Pycharm ? ;-(
-Node = TypeVar("Node", bound=ComparableAndHashable)
-DirectedEdge = Tuple[Node, Node]
-UndirectedEdge = FrozenSet[Node]
-Edge = Union[DirectedEdge, UndirectedEdge]
-EdgeLike = Union[Edge, Set[Node], Iterable[Node]]
-Label = Any
-InternalGraphRepresentation = Dict[Node, Dict[Node, Union[int, List[Label]]]]
+from smallgraphlib.custom_types import _AbstractGraph, Node, Edge, EdgeLike
+from smallgraphlib.utilities import cached_property, Multiset, clear_cache
+from smallgraphlib.tikz_export import tikz_printer
 
 
 class Traversal(Enum):
@@ -84,7 +67,7 @@ class AbstractGraph(ABC, Generic[Node]):
     # ------------------
 
     @classmethod
-    def from_string(cls, string: str):
+    def from_string(cls: Type[_AbstractGraph], string: str) -> _AbstractGraph:
         """DirectedGraph.from_string("A:B,C B:C C") will generate a graph of 3 nodes, A, B and C, with
         edges A->B, A->C and B->C."""
         nodes: List[str] = []
@@ -94,7 +77,7 @@ class AbstractGraph(ABC, Generic[Node]):
             nodes.append(node.strip())
             if remaining:
                 edges.extend((node, successor.strip()) for successor in remaining[0].split(","))
-        return cls(nodes, *edges)  # type: ignore
+        return cls(nodes, *edges)
 
     @staticmethod
     def _matrix_as_tuple_of_tuples(matrix: Iterable[Iterable]) -> Tuple[Tuple, ...]:
@@ -116,8 +99,8 @@ class AbstractGraph(ABC, Generic[Node]):
 
     @classmethod
     def from_matrix(
-        cls, matrix: Iterable[Iterable[int]], nodes_names: Iterable[Node] = None
-    ) -> "AbstractGraph":
+        cls: Type[_AbstractGraph], matrix: Iterable[Iterable[int]], nodes_names: Iterable[Node] = None
+    ) -> _AbstractGraph:
         """Construct the graph corresponding to the given adjacency matrix.
 
         Matrix must be a matrix of positive integers
@@ -686,21 +669,21 @@ class AbstractGraph(ABC, Generic[Node]):
                 f"Order must be Traversal.PREORDER, Traversal.POSTORDER or Traversal.INORDER, not {order!r}."
             )
 
-        def preorder_dfs(node):
+        def preorder_dfs(node: Node) -> Iterator[Node]:
             if node not in visited:
                 yield node
                 visited.add(node)
                 for successor in self._successors[node]:
                     yield from preorder_dfs(successor)
 
-        def postorder_dfs(node):
+        def postorder_dfs(node: Node) -> Iterator[Node]:
             if node not in visited:
                 visited.add(node)
                 for successor in self._successors[node]:
                     yield from postorder_dfs(successor)
                 yield node
 
-        def inorder_dfs(node):
+        def inorder_dfs(node: Node) -> Iterator[Node]:
             visited.add(node)
             # Eliminate visited nodes from the list of successors *before* splitting it.
             successors: List[Node] = [
@@ -750,6 +733,14 @@ class AbstractGraph(ABC, Generic[Node]):
             yield node
             queue.extend(successor for successor in self._successors[node] if successor not in visited)
 
+    def _tikz_specific_node_style(self, node: Node) -> str:
+        """Overwrite this method to add a specific tikz style to some nodes."""
+        return ""
+
+    def _tikz_labels(self, node1: Node, node2: Node) -> list[str]:
+        """Overwrite this method to modify tikz value for some labels."""
+        return self.labels(node1, node2)
+
     def as_tikz(self, *, shuffle_nodes=False, options="") -> str:
         r"""Generate tikz code corresponding to this graph.
 
@@ -764,148 +755,7 @@ class AbstractGraph(ABC, Generic[Node]):
             \contourlength{0.5pt}
 
         """
-        lines: List[str] = [
-            r"\providecommand{\contour}[2]{#2}"  # avoid an error if package contour is not loaded.
-            r"\begin{tikzpicture}["
-            r"every node/.style = {font={\scriptsize}},"
-            r"vertex/.style = {draw, circle,font={\scriptsize},inner sep=2},"
-            "directed/.style = {-{Stealth[scale=1.1]}},"
-            "reversed/.style = {{Stealth[scale=1.1]}-},"
-            "undirected/.style = {},"
-            f"{options}"
-            "]"
-        ]
-        theta = 360 / self.order
-        angles = {}
-        nodes = list(self.nodes)
-        if shuffle_nodes:
-            random.shuffle(nodes)
-        index: Dict[Node, int] = {node: i for i, node in enumerate(nodes)}
-        # All nodes are placed around a circle, creating a regular polygon.
-        for i, node in enumerate(nodes):
-            angle = angles[node] = i * theta
-            lines.append(rf"\node[vertex] ({node}) at ({angle}:1cm) {{${node}$}};")
-
-        # Store nodes' cartesian coordinates.
-        nodes_positions: Dict[Node, Tuple[float, float]] = {}
-        for node in nodes:
-            alpha = math.radians(angles[node])
-            nodes_positions[node] = math.cos(alpha), math.sin(alpha)
-
-        def label_position(node1_, node2_, k):
-            """Return label cartesian coordinates for position `k`.
-
-            Position `k` is a barycentric coefficient between 0 and 1."""
-            x1, y1 = nodes_positions[node1_]
-            x2, y2 = nodes_positions[node2_]
-            return k * x2 + (1 - k) * x1, k * y2 + (1 - k) * y1
-
-        # Store the places already occupied by labels, to avoid storing another label there.
-        labels_positions: List[Tuple[float, float]] = []
-        # Detect edges' intersections, to avoid positioning labels there.
-        for node1 in nodes:
-            for node2 in self.successors(node1) | self.predecessors(node1):
-                if index[node1] < index[node2]:
-                    for node3 in nodes:
-                        for node4 in self.successors(node3) | self.predecessors(node3):
-                            if index[node3] < index[node4] and len({node1, node2, node3, node4}) == 4:
-                                A, B, C, D = (nodes_positions[node] for node in (node1, node2, node3, node4))
-                                intersection = segments_intersection((A, B), (C, D))
-                                if intersection is not None:
-                                    labels_positions.append(intersection)
-
-        for node1 in nodes:
-            # If the graph is undirected, draw only i -> j edge and not j -> i edge,
-            # since it is in fact the same edge.
-            # An easy way to do that is to keep index[node2] always superior or equal to index[node1].
-            for node2 in nodes if self.is_directed else nodes[index[node1] :]:
-                if node1 == node2:
-                    # This is a loop.
-                    node = node1
-                    style = "directed" if self.is_directed else "undirected"
-                    n: int = self.count_edges(node, node, count_undirected_loops_twice=False)
-                    if n > _TIKZ_EXPORT_MAX_MULTIPLE_LOOPS_SUPPORT:
-                        raise NotImplementedError(n)
-                    if n == 1:
-                        (label,) = self.labels(node, node)
-                        lines.append(
-                            rf"\draw[{style}] ({node}) to "
-                            f"[out={angles[node] - 45},in={angles[node] + 45},looseness=5] "
-                            rf"node[midway] {{\contour{{white}}{{{label}}}}} "
-                            f"({node});"
-                        )
-                else:
-                    # This is a normal edge, joining two different nodes.
-                    styles: List[str] = []
-                    labels: List[str] = []
-                    # Detect if node1 and node2 are neighbours on the circle.
-                    node2_is_right_neighbour = (index[node1] - index[node2] - 1) % len(nodes) == 0
-                    node2_is_left_neighbour = (index[node1] - index[node2] + 1) % len(nodes) == 0
-
-                    if self.is_directed:
-                        data = [("directed", node1, node2), ("reversed", node2, node1)]
-                    else:
-                        data = [("undirected", node1, node2)]
-                    for direction, nodeA, nodeB in data:
-                        labels.extend(self.labels(nodeA, nodeB))
-                        styles += self.count_edges(nodeA, nodeB) * [direction]
-                    n = len(styles)
-                    assert len(labels) == n, f"len(styles)={n} != len(labels)={len(labels)}"
-                    if n == 0:
-                        bendings = []
-                    elif n == 1:
-                        bendings = [""]  # strait line by default
-                        if len(nodes) >= 6:
-                            if node2_is_left_neighbour:
-                                bendings[0] = "bend right=30"
-                            elif node2_is_right_neighbour:
-                                bendings[0] = "bend left=30"
-                    elif n == 2:
-                        bendings = ["bend left=15", "bend right=15"]
-                    elif n == 3:
-                        bendings = ["bend left", "", "bend right"]
-                    else:
-                        assert n > _TIKZ_EXPORT_MAX_MULTIPLE_EDGES_SUPPORT
-                        raise NotImplementedError(
-                            f"Too much multiple edges : {n} > {_TIKZ_EXPORT_MAX_MULTIPLE_EDGES_SUPPORT} "
-                            f"for graph {self}."
-                        )
-                    for style, bending, label in zip(styles, bendings, labels):
-                        label_tikz_code = ""
-                        if label:
-                            pos = 0.5
-                            if node2_is_right_neighbour or node2_is_left_neighbour:
-                                pass
-                            else:
-                                # Try to minimize collisions between two labels.
-                                # This dict will store the distance between the nearest labels
-                                # for each position.
-                                min_dists: Dict[float, float] = {}
-                                # This one will store the coordinates of the label for each position.
-                                coordinates: Dict[float, Tuple[float, float]] = {}
-                                for pos in (
-                                    0.1,
-                                    0.2,
-                                    0.3,
-                                    0.4,
-                                    0.5,
-                                    0.6,
-                                    0.7,
-                                    0.8,
-                                    0.9,
-                                ):
-                                    new_x, new_y = label_position(node1, node2, pos)
-                                    min_dists[pos] = min(
-                                        (new_x - x) ** 2 + (new_y - y) ** 2 for (x, y) in labels_positions
-                                    )
-                                    coordinates[pos] = new_x, new_y
-                                pos = max(min_dists, key=min_dists.get)  # type: ignore
-                                labels_positions.append(coordinates[pos])
-                            label_tikz_code = rf"node[pos={pos}] {{\contour{{white}}{{{label}}}}}"
-                        lines.append(rf"\draw[{style}] ({node1}) to[{bending}] {label_tikz_code} ({node2});")
-
-        lines.append(r"\end{tikzpicture}")
-        return "\n".join(lines)
+        return tikz_printer.tikz_code(self, shuffle_nodes=shuffle_nodes, options=options)
 
 
 class InvalidGraphAttribute(AttributeError):
